@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-'''
+"""
 Created on Mar 25, 2015
 
 @author: Gaurav Rastogi (grastogi@avinetworks.com)
@@ -15,7 +15,7 @@ Pool:
       name: grastogi
       cloud: AWS
 
-'''
+"""
 import argparse
 import json
 import logging
@@ -24,6 +24,7 @@ import traceback
 
 import yaml
 import re
+import collections
 from copy import deepcopy
 from avi.migrationtools.avi_migration_utils import MigrationUtil
 
@@ -51,23 +52,28 @@ class ConfigPatch(object):
         :param param_name:
         :return:
         """
-        qp = avi_ref.split('?')[1]
-        params = qp.split('&')
-        for param in params:
-            k, v = param.split('=')
-            if k == param_name:
-                return v
+        try:
+            qp = avi_ref.split('?')[1]
+            params = qp.split('&')
+            for param in params:
+                k, v = param.split('=')
+                if k == param_name:
+                    return v
+        except:
+            return ""
         log.error('returning param %s %s', avi_ref, param_name)
         raise Exception('Could not find param %s in ref %s' %
                         (param_name, avi_ref))
 
-    def update_obj_refs(self, old_obj_type, old_ref, new_ref, obj):
+    def update_obj_refs(self, old_obj_type, old_ref, new_ref, obj, avi_cfg):
         """
         Traverses the full object maps and updates the references
+        :param old_obj_type: object type for the original object.
         :param old_ref: old reference
         :param new_ref: new reference
         :param obj: Object dictionary or list. It could be nested part of the
         object as well.
+        :param avi_cfg: Full Avi configuration dictionary.
         :return: None
         """
         if isinstance(obj, dict):
@@ -86,14 +92,14 @@ class ConfigPatch(object):
                         obj[k] = new_ref
                 elif isinstance(v, dict):
                     self.update_obj_refs(
-                        old_obj_type, old_ref, new_ref, v)
+                        old_obj_type, old_ref, new_ref, v, avi_cfg)
                 elif isinstance(v, list):
                     for elem in v:
                         self.update_obj_refs(
-                            old_obj_type, old_ref, new_ref, elem)
+                            old_obj_type, old_ref, new_ref, elem, avi_cfg)
         elif isinstance(obj, list):
             for elem in obj:
-                self.update_references(old_ref, new_ref, elem)
+                self.update_references(old_ref, new_ref, elem, avi_cfg)
         else:
             # ignores the object.
             pass
@@ -111,7 +117,30 @@ class ConfigPatch(object):
         for _, obj_list in avi_cfg.iteritems():
             for obj in obj_list:
                 self.update_obj_refs(
-                    obj_type, old_ref, new_ref, obj)
+                    obj_type, old_ref, new_ref, obj, avi_cfg)
+
+    def update_tenant_references(self, avi_config, old_tenant, new_tenant):
+        for obj_type in avi_config.keys():
+            if obj_type == 'META':
+                avi_config[obj_type]['use_tenant'] = new_tenant
+                continue
+            for obj in avi_config[obj_type]:
+                for key in obj:
+                    if key == 'tenant_ref':
+                        obj[key] = '/api/tenant/?name=%s' % new_tenant
+                    if key.endswith('ref') and not key == 'tenant_ref':
+                        if not obj[key]:
+                            continue
+                        obj[key] = obj[key].replace(
+                            'tenant=%s' % old_tenant, 'tenant=%s' % new_tenant)
+                    elif key.endswith('refs'):
+                        new_refs = []
+                        for ref in obj[key]:
+                            new_refs.append(ref.replace(
+                                'tenant=%s' % old_tenant,
+                                'tenant=%s' % new_tenant))
+                        obj[key] = new_refs
+        return
 
     def apply_obj_patch(self, obj_type, obj, patch_data, avi_cfg):
         """
@@ -122,10 +151,8 @@ class ConfigPatch(object):
         :return:
         """
         old_obj_refs = []
+        mg_util = MigrationUtil()
 
-        # TODO(grastogi): The refs computation needs to change
-        # as the currently the ref does not have tenant or
-        # cloud information.
         tenant_ref = obj.get('tenant_ref', '/api/tenant/?name=admin')
         tenant = self.param_value_in_ref(tenant_ref, 'name')
         if 'name' in obj:
@@ -134,34 +161,58 @@ class ConfigPatch(object):
             old_obj_refs.append(old_obj_ref)
             old_obj_ref = obj['name']
             old_obj_refs.append(old_obj_ref)
-            old_obj_ref = '/api/%s/?tenant=%s&name=%s' % (
-                obj_type.lower(), tenant, obj['name'])
-
+            # old_obj_ref = '/api/%s/?tenant=%s&name=%s' % (
+            #     obj_type.lower(), tenant, obj['name'])
             cloud = (self.param_value_in_ref(obj.get('cloud_ref'), 'name')
-                     if 'cloud_ref' in obj else '')
-            if cloud:
-                old_obj_ref = '%s&cloud=%s' % (old_obj_ref, cloud)
+                     if 'cloud_ref' in obj else None)
+            old_obj_ref = mg_util.get_object_ref(
+                obj['name'], obj_type.lower(), tenant, cloud_name=cloud)
 
             old_obj_refs.append(old_obj_ref)
-        log.debug('patching %s:%s with patch %s',
-                  obj_type, obj.get('name', ''), patch_data)
-        obj.update(patch_data['patch'])
-        if 'name' in obj:
-            new_obj_name = obj['name']
+        if patch_data.get('delete_old', False):
+            new_obj_name = patch_data['patch']['name']
+            avi_cfg[obj_type].remove(obj)
+            log.debug('Deleted old %s named %s replacing references to %s',
+                      obj_type, obj.get('name', ''), new_obj_name)
+            if 'tenant_ref' in patch_data['patch']:
+                tenant = mg_util.get_name(patch_data['patch']['tenant_ref'])
+
+        else:
+            log.debug('patching %s:%s with patch %s',
+                      obj_type, obj.get('name', ''), patch_data)
+            self.deep_update(obj, patch_data['patch'])
+            if 'name' in obj:
+                new_obj_name = obj['name']
         if (('name' in patch_data['patch']) and old_obj_refs and
                 (obj_name != new_obj_name)):
             # need to update all the references for this object
-            new_obj_ref = '/api/%s/?tenant=%s&name=%s' % (
-                obj_type.lower(), tenant, obj['name'])
+            # new_obj_ref = '/api/%s/?tenant=%s&name=%s' % (
+            #     obj_type.lower(), tenant, obj['name'])
             cloud = (self.param_value_in_ref(obj.get('cloud_ref'), 'name')
                      if 'cloud_ref' in obj else '')
-            if cloud:
-                new_obj_ref += '&cloud=%s' % cloud
+            new_obj_ref = mg_util.get_object_ref(
+                new_obj_name, obj_type.lower(), tenant, cloud_name=cloud)
             # this is to handle old references could be in multiple formats
             for old_obj_ref in old_obj_refs:
                 self.update_references(
                     obj_type, old_obj_ref, new_obj_ref, avi_cfg)
 
+    def deep_update(self, d, u):
+        for k, v in u.iteritems():
+            if isinstance(v, collections.Mapping):
+                d[k] = self.deep_update(d.get(k, {}), v)
+            elif isinstance(v, list):
+                for i in v:
+                    cnt = 0
+                    if isinstance(i, collections.Mapping):
+                        d[k][cnt] = self.deep_update(d[k][cnt], i)
+                    else:
+                        d[k][cnt] = i
+                    cnt += 1
+            else:
+                d[k] = v
+        return d
+
     def apply_patch(self, obj_type, patch_data, new_cfg):
         """
         For every patch in the patch data
@@ -174,19 +225,28 @@ class ConfigPatch(object):
         :return:
         """
         cfg_objs = new_cfg.get(obj_type, [])
+        if obj_type == 'Tenant' and 'match_name' in patch_data \
+                and 'name' in patch_data['patch']:
+            self.update_tenant_references(
+                new_cfg, patch_data['match_name'], patch_data['patch']['name'])
         if not cfg_objs:
             log.warn('Could not apply patch %s: %s as no matching obj found',
                      obj_type, patch_data)
             return new_cfg
         for obj in cfg_objs:
             obj_name = obj['name']
-            regex_pattern = '.*'
+            rexp = None
+            list_match = False
             if 'match_name' in patch_data:
                 regex_pattern = '^%s$' % patch_data['match_name']
+                rexp = re.compile(regex_pattern)
             elif 'match_name_regex' in patch_data:
                 regex_pattern = patch_data['match_name_regex']
-            rexp = re.compile(regex_pattern)
-            if rexp.match(obj_name):
+                rexp = re.compile(regex_pattern)
+            elif 'match_name_in_list' in patch_data:
+                list_match = obj_name in patch_data['match_name_in_list']
+
+            if (rexp and rexp.match(obj_name)) or list_match:
                 try:
                     self.apply_obj_patch(obj_type, obj, patch_data, new_cfg)
                 except:
@@ -216,7 +276,7 @@ class ConfigPatch(object):
                 self.apply_patch(obj_type, patch_data, new_cfg)
             msg = "Patching conversion started..."
             mg_util.print_progress_bar(progressbar_count, total_size, msg,
-                               prefix='Progress', suffix='')
+                                       prefix='Progress', suffix='')
         return new_cfg
 
 
@@ -231,7 +291,8 @@ if __name__ == '__main__':
     root_logger.addHandler(ch)
 
     parser = argparse.ArgumentParser(
-        description='Patches an exported Avi Configuration with a configuration patch input as yaml file.',
+        description='Patches an exported Avi Configuration with a '
+                    'configuration patch input as yaml file.',
         usage="""
     python config_patch -c avi_config.json -p test/patch.yml
     output: avi_config.json.patched
@@ -244,18 +305,23 @@ if __name__ == '__main__':
       - match_name: cool
         patch:
           name: awesome
-        """)
+
+    Please ensure configuration is exported with options include_name=true&uuid_refs=false as:
+    Example:
+        api/configuration/export?include_name=true&uuid_refs=true&uuid_refs=false
+    """)
     parser.add_argument('-c', '--aviconfig',
                         help='Avi configuration in JSON format')
     parser.add_argument('-p', '--patchconfig',
-                        help='Avi configuration objects to be patched. It is list of patterns and object overrides')
+                        help='Avi configuration objects to be patched. '
+                             'It is list of patterns and object overrides')
     args = parser.parse_args()
 
     with open(args.aviconfig) as f:
         acfg = json.load(f)
 
     with open(args.patchconfig) as f:
-        patches = yaml.load(f)
+        patches = yaml.load(f, Loader=yaml.Loader)
     cp = ConfigPatch(acfg, patches)
     patched_cfg = cp.patch()
     with open(args.aviconfig + '.patched', 'w') as f:
