@@ -179,18 +179,6 @@ class PoolConfigConv(object):
         if limits.get('connection_limit', 0) > 0:
             pool_obj['max_concurrent_connections_per_server'] = \
                 limits['connection_limit']
-        if limits.get('rate_limit', 0) > 0:
-            pool_obj['max_conn_rate_per_server'] = {
-                'count': limits['rate_limit'],
-                'explicit_tracking': False,
-                'period': 1,
-                'action': {
-                    'status_code': 'HTTP_LOCAL_RESPONSE_STATUS_CODE_429',
-                    'type': "RL_ACTION_NONE"
-                },
-                'burst_sz': 0,
-                'fine_grain': False
-            }
         return pool_obj
 
     def check_for_pool_group(self, servers):
@@ -379,7 +367,7 @@ class PoolConfigConvV11(PoolConfigConv):
                 "enabled": True
             }
             pool_obj['server_reselect'] = server_reselect
-        monitor_names = f5_pool.get("monitor", None)
+        monitor_names = f5_pool.get("monitor ", None)
         skipped_monitors = []
         if monitor_names:
             skipped_monitors, monitor_refs = super(
