@@ -12,6 +12,7 @@ from avi.migrationtools.netscaler_converter.ns_constants \
 from avi.migrationtools.netscaler_converter.monitor_converter \
     import merge_object_mapping
 from avi.migrationtools.netscaler_converter.ns_util import NsUtil
+from avi.migrationtools.avi_migration_utils import update_count
 app_per_merge_count = {'count': 0}
 
 LOG = logging.getLogger(__name__)
@@ -151,6 +152,7 @@ class ServiceConverter(object):
                         LOG.warning(skipped_status)
                     continue
                 ns_algo = lb_vs.get('lbMethod', 'LEASTCONNECTIONS')
+                service_type = lb_vs['attrs'][1]
                 algo = ns_util.get_avi_lb_algorithm(ns_algo)
                 pg_members = []
                 for element in group:
@@ -174,10 +176,27 @@ class ServiceConverter(object):
                                 pool_name, group_key, avi_config,
                                 userprefix=self.prefix)
                         pool[0]['lb_algorithm'] = algo
+                        if not pool[0].get('health_monitor_refs') and not \
+                                pool[0].get('use_service_port', False):
+                            if service_type == 'TCP':
+                                pool[0]['health_monitor_refs'] = [
+                                    ns_util.get_object_ref(
+                                        'tcp-default',
+                                        OBJECT_TYPE_HEALTH_MONITOR,
+                                        self.tenant_name)]
+                            else:
+                                pool[0]['health_monitor_refs'] = [
+                                    ns_util.get_object_ref(
+                                        'ping-default',
+                                        OBJECT_TYPE_HEALTH_MONITOR,
+                                        self.tenant_name)]
                         updated_pool_ref = ns_util.get_object_ref(
                             pool_name, OBJECT_TYPE_POOL, self.tenant_name,
                             self.cloud_name)
-                        pg_members.append({'pool_ref': updated_pool_ref})
+                        pg_members.append({'pool_ref': updated_pool_ref,
+                                           'ratio': 1,
+                                           'priority_label': '10'
+                                           })
                         used_pool_ref.append(pool_name)
                         LOG.info('Conversion successful : %s' % full_cmd)
                         # Add summery of add server in CSV/report
@@ -211,6 +230,7 @@ class ServiceConverter(object):
                     avi_config['PoolGroup'].append(pool_group)
 
             except Exception as e:
+                update_count('error')
                 LOG.error('Error in bind lb vserver conversion bound to: %s' %
                           group_key, exc_info=True)
             msg = "PoolGroup Conversion started..."
@@ -450,8 +470,7 @@ class ServiceConverter(object):
                 elif service.get('attrs') and len(service['attrs']) >= 2 and \
                         'SSL' in service['attrs']:
                     pool_obj['ssl_profile_ref'] = ns_util.get_object_ref(
-                            'System-Standard', OBJECT_TYPE_SSL_PROFILE,
-                            self.tenant_name)
+                            'System-Standard', OBJECT_TYPE_SSL_PROFILE, 'admin')
                 # Remove health monitor reference of http type if pool
                 # has ssl profile or pki profile or ssl cert key
                 # ELSE remove health monitor of https type
@@ -487,6 +506,7 @@ class ServiceConverter(object):
                     service['line_no'], service_command, service_name,
                     service_netscalar_full_command, conv_status, pool_obj)
             except:
+                update_count('error')
                 LOG.error('Error in pool conversion for: %s' % key,
                           exc_info=True)
             # Calling progressbar function.
@@ -611,8 +631,7 @@ class ServiceConverter(object):
                 elif (service_group.get('attrs') and len(service_group['attrs'])
                         >= 2 and 'SSL' in service_group['attrs']):
                     pool_obj['ssl_profile_ref'] = ns_util.get_object_ref(
-                        'System-Standard', OBJECT_TYPE_SSL_PROFILE,
-                        self.tenant_name)
+                        'System-Standard', OBJECT_TYPE_SSL_PROFILE, 'admin')
                 # Remove health monitor reference of http type if pool
                 # has ssl profile or pki profile or ssl cert key
                 # ELSE remove health monitor of https type
@@ -654,6 +673,7 @@ class ServiceConverter(object):
                     service_group_name, service_group_netscalar_full_command,
                     conv_status, pool_obj)
             except:
+                update_count('error')
                 LOG.error('Error in pool conversion for: %s' % group_key,
                           exc_info=True)
             # Calling progress bar function.
@@ -915,6 +935,7 @@ class ServiceConverter(object):
                     ip_addr = socket.gethostbyname(ip_addr)
                     server_obj['ip']['addr'] = ip_addr
                 except:
+                    update_count('warning')
                     # Skipped this server if it does not have an Ip
                     ns_util.add_status_row(
                         server['line_no'], ns_add_server_command, server['attrs'][0],
