@@ -10,6 +10,7 @@ from avi.migrationtools.netscaler_converter.ns_constants \
     import (STATUS_SKIPPED, STATUS_SUCCESSFUL, STATUS_DATASCRIPT,
             OBJECT_TYPE_POOL_GROUP, OBJECT_TYPE_STRING_GROUP)
 from avi.migrationtools.netscaler_converter.ns_util import NsUtil
+from avi.migrationtools.avi_migration_utils import update_count
 
 
 LOG = logging.getLogger(__name__)
@@ -51,7 +52,6 @@ class PolicyConverter(object):
         # Added prefix for objects
         self.prefix = prefix
 
-
     def convert(self, bind_conf_list, ns_config, avi_config, tmp_pool_ref,
                 redirect_pools, netscalar_command, case_sensitive, enable_ssl):
         """
@@ -69,6 +69,7 @@ class PolicyConverter(object):
         """
         policy_lables = ns_config.get('bind cs policylabel', {})
         policy_config = ns_config.get('add cs policy', {})
+        cs_action_config = ns_config.get('add cs action', {})
         responder_policy_config = ns_config.get('add responder policy', {})
         rewrite_policy_config = ns_config.get('add rewrite policy', {})
         responder_action_config = ns_config.get('add responder action', {})
@@ -76,6 +77,7 @@ class PolicyConverter(object):
         bind_patset = ns_config.get('bind policy patset', {})
         patset_config = ns_config.get('add policy patset', {})
         policy_expression_config = ns_config.get('add policy expression', {})
+
         http_request_policy = {
             'rules': []
         }
@@ -161,6 +163,15 @@ class PolicyConverter(object):
                     policy_name, policy_config, rewrite_policy_config,
                     responder_policy_config)
 
+                if policy and not targetLBVserver and 'action' in policy:
+                    cs_action = cs_action_config[policy['action']]
+                    targetLBVserver = cs_action.get('targetLBVserver', None)
+                    cs_action_cmd = ns_util.get_netscalar_full_command(
+                        'add cs action', cs_action)
+                    ns_util.add_status_row(
+                        cs_action['line_no'], 'add cs action',
+                        cs_action['attrs'][0], cs_action_cmd, STATUS_SUCCESSFUL)
+
                 if not policy:
                     skipped_status = 'Skipped: Policy is not created %s' \
                                      % bind_lb_netscalar_complete_command
@@ -223,6 +234,7 @@ class PolicyConverter(object):
                         STATUS_SKIPPED, skipped_status)
                     continue
             except:
+                update_count('error')
                 LOG.error('Error in policy conversion', exc_info=True)
         if len(http_request_policy['rules']) > 0:
             policy_obj['http_request_policy'] = http_request_policy
@@ -267,7 +279,6 @@ class PolicyConverter(object):
                 self.get_targetvserver_policylabel(
                     self, policyLabels, policy_lables, depth=depth-1)
 
-
     def rule_converter(self, policy, policy_type, priority_index,
                        redirect_pools, bind_patset, patset_config,
                        rewrite_action_config, responder_action_config,
@@ -294,6 +305,8 @@ class PolicyConverter(object):
         netscalar_command = None
         rule_name = policy['attrs'][0]
         ns_rule = policy.get('rule', None)
+        if not ns_rule and 'url' in policy:
+            ns_rule = 'URL ==%s' % policy['url']
         if policy_type == 'cs':
             netscalar_command = 'add cs policy'
         elif policy_type == 'rewrite':
@@ -456,7 +469,6 @@ class PolicyConverter(object):
             priority_index = priority_index + 1
         return policy_rules, priority_index
 
-
     def query_converter(self, rule, policy_name, bind_patset, patset_config,
                         avi_config, case_sensitive):
         """
@@ -506,9 +518,7 @@ class PolicyConverter(object):
         }
 
         if 'true' == query.lower():
-            match = {"path": path_query}
-            match["path"]["match_str"].append('/')
-            match["path"]["match_criteria"] = "CONTAINS"
+            match = {'any': 'any'}
 
         elif 'URL ==' in query.upper() or 'REQ.HTTP.URL ==' in query.upper():
             a, b = query.split("==")
@@ -521,10 +531,11 @@ class PolicyConverter(object):
             match["path"]["match_str"].append(match_str)
             match["path"]["match_criteria"] = "EQUALS"
 
-        elif 'HTTP.REQ.URL.PATH_AND_QUERY.CONTAINS' in query.upper() or \
-                        'HTTP.REQ.URL.QUERY.CONTAINS' in query.upper() or \
-                        'HTTP.REQ.URL.PATH.STARTSWITH' in query.upper() or \
-                        'HTTP.REQ.URL.STARTSWITH' in query.upper():
+        elif ('HTTP.REQ.URL.PATH_AND_QUERY.CONTAINS' in query.upper() or
+              'HTTP.REQ.URL.QUERY.CONTAINS' in query.upper() or
+              'HTTP.REQ.URL.PATH.STARTSWITH' in query.upper() or
+              'HTTP.REQ.URL.STARTSWITH' in query.upper() or
+              'HTTP.REQ.URL.CONTAINS' in query.upper()):
             match = {"query": path_query}
             match["query"]["match_criteria"] = "QUERY_MATCH_CONTAINS"
             matches = re.findall('\\\\(.+?)\\\\', query)
@@ -566,6 +577,18 @@ class PolicyConverter(object):
                 match["client_ip"]["addrs"].append(
                     {"type": 'V4', "addr": element})
 
+        elif ('HTTP.REQ.HEADER' in query.upper() and \
+              '.CONTAINS' in query.upper()) and '.NOT' in query.upper():
+            match = {"hdrs": [header]}
+            match["hdrs"][0]["match_criteria"] = "HDR_DOES_NOT_CONTAIN"
+
+            matches = re.findall('\\\\(.+?)\\\\', query)
+            if len(matches) < 2 or matches[0] is None or matches[1] is None:
+                LOG.warning('No Matches found for %s' % query)
+                return None
+            match["hdrs"][0]["hdr"] = matches[0]
+            match["hdrs"][0]["value"].append(matches[1])
+
         elif ('HTTP.REQ.HEADER' in query.upper() and \
                           '.CONTAINS' in query.upper()) or \
                         'HTTP.REQ.FULL_HEADER.CONTAINS' in query.upper():
@@ -604,6 +627,30 @@ class PolicyConverter(object):
                 element = re.sub('[\\\/]', '', element)
                 match["host_hdr"]["value"].append(element)
 
+        elif ('HTTP.REQ.HOSTNAME.CONTAINS' in query.upper() or
+              ('HTTP.REQ.HOSTNAME.DOMAIN.SET_TEXT_MODE' in query.upper() and
+              'CONTAINS' in query.upper())):
+            match = {"host_hdr": host_header}
+            match["host_hdr"]["match_criteria"] = "HDR_CONTAINS"
+            matches = re.findall('\\\\(.+?)\\\\', query)
+            if len(matches) == 0:
+                LOG.warning('No Matches found for %s' % query)
+                return None
+            for element in matches:
+                element = re.sub('[\\\/]', '', element)
+                match["host_hdr"]["value"].append(element)
+
+        elif ('HTTP.REQ.HOSTNAME.STARTSWITH' in query.upper()):
+            match = {"host_hdr": host_header}
+            match["host_hdr"]["match_criteria"] = "HDR_BEGINS_WITH"
+            matches = re.findall('\\\\(.+?)\\\\', query)
+            if len(matches) == 0:
+                LOG.warning('No Matches found for %s' % query)
+                return None
+            for element in matches:
+                element = re.sub('[\\\/]', '', element)
+                match["host_hdr"]["value"].append(element)
+
         elif ('HTTP.REQ.COOKIE' in query.upper()
               and 'CONTAINS' in query.upper()) or \
                 ('HTTP.REQ.COOKIE' in query.upper() and 'EQ(' in query.upper()):
@@ -649,6 +696,25 @@ class PolicyConverter(object):
                 string_group_ref, OBJECT_TYPE_STRING_GROUP, self.tenant_name)
             match["path"]["string_group_refs"].append(updated_string_group_ref)
 
+        elif ('HTTP.REQ.URL.PATH' in query.upper() and
+              'REGEX_MATCH' in query.upper()):
+            match = {"path": path_regex}
+            match["path"]["match_criteria"] = "REGEX_MATCH"
+            regex_match = None
+            if 're#' in query:
+                regex_match = query.split('re#')[1].split('#')[0]
+            if not regex_match:
+                LOG.warning(
+                    "%s Rule PATH for regex match did not start with 're#'" %
+                    query)
+
+            string_group_ref = self.add_string_group_for_policy(
+                '%s-string_group_object' % policy_name, [regex_match],
+                avi_config)
+            updated_string_group_ref = ns_util.get_object_ref(
+                string_group_ref, OBJECT_TYPE_STRING_GROUP, self.tenant_name)
+            match["path"]["string_group_refs"].append(updated_string_group_ref)
+
         elif 'HTTP.REQ.URL.PATH.GET' in query.upper() \
                 and 'EQ(' in query.upper():
             match = {"path": path_query}
@@ -745,7 +811,8 @@ class PolicyConverter(object):
                 element = re.sub('[\\\/]', '', element)
                 match["path"]["match_str"].append(element)
 
-        elif 'HTTP.REQ.URL.EQ' in query.upper():
+        elif ('HTTP.REQ.URL.EQ' in query.upper() or
+              'HTTP.REQ.URL.CONTAINS' in query.upper()):
             match = {"query": path_query}
             match["query"]["match_criteria"] = "QUERY_MATCH_CONTAINS"
 
@@ -779,22 +846,38 @@ class PolicyConverter(object):
         elif 'HTTP.REQ.IS_VALID' in query.upper():
             match = {'any': 'any'}
 
+        elif ('HTTP.REQ.URL.SET_TEXT_MODE' in query.upper() and 'STARTSWITH_ANY' in query.upper()):
+            match = {"path": path_query}
+            match["path"]["match_criteria"] = "BEGINS_WITH"
+            match["path"]["match_case"] = "INSENSITIVE"
+            matches = re.findall('\\\\"(.+?)\\\\', query)
+            if len(matches) == 0:
+                LOG.warning('No Matches found for %s' % query)
+                return None
+            patsets = []
+            for element in matches:
+                element = re.sub('[\\\/]', '', element)
+                patset = self.get_patset_collection(element, bind_patset,
+                                                    patset_config)
+                if patset:
+                    patsets += patset
+            match["path"]["match_str"] = patsets
+
         else:
             LOG.warning("%s Rule is not supported" % query)
             return None
         if match:
-            if match.get('path', None) and \
-                            match['path']['match_str'][0] == '/*':
+            if ('path' in match and 'match_str' in match['path'] and
+                    match['path']['match_str'][0] == '/*'):
                 match['path']['match_str'][0] = '/'
                 match['path']['match_criteria'] = 'CONTAINS'
-            elif match.get('path', None) and \
-                    match['path']['match_str'][0].endswith('*'):
+            elif('path' in match and 'match_str' in match['path'] and
+                 match['path']['match_str'][0].endswith('*')):
                 match['path']['match_criteria'] = 'BEGINS_WITH'
                 match['path']['match_str'][0] = \
                     match['path']['match_str'][0][:-1]
             return match
 
-
     def add_string_group_for_policy(self, string_group_name, matches,
                                     avi_config):
         """
@@ -810,6 +893,7 @@ class PolicyConverter(object):
         stringgroup_object = {
             "name": string_group_name,
             "kv": [],
+            "type": "SG_TYPE_STRING",
             "tenant_ref": self.tenant_ref
         }
 
@@ -828,14 +912,29 @@ class PolicyConverter(object):
         :param patset_config: dict patset config
         :return: patset attributes
         """
+        netscalar_command = 'bind policy patset'
+
 
         if match in patset_config and match in bind_patset:
             patsets = bind_patset[match]
             patset_attrs = []
+            patsets = sorted(patsets, key = lambda patset: int(patset['index']))
             for patset in patsets:
-                attrs = [x for x in patset['attrs'] if x != match]
+                ns_policy_complete_cmd = \
+                    ns_util.get_netscalar_full_command(netscalar_command, patset)
+                attrs = [re.sub(r'^"|"$', '', x) for x in patset['attrs'] if x != match]
                 patset_attrs += attrs
+                ns_util.add_status_row(
+                    patset['line_no'], netscalar_command, match,
+                    ns_policy_complete_cmd, STATUS_SUCCESSFUL, attrs)
             if patset_attrs:
+                netscalar_command = 'add policy patset'
+                ns_policy_complete_cmd = \
+                    ns_util.get_netscalar_full_command(netscalar_command, patset_config[match])
+                ns_util.add_status_row(
+                    patset_config[match]['line_no'], netscalar_command, match,
+                    ns_policy_complete_cmd, STATUS_SUCCESSFUL, patset_attrs)
+
                 return patset_attrs
 
         LOG.warning("%s Patset policy is not supported" % match)
@@ -914,8 +1013,8 @@ class PolicyConverter(object):
             policy_rule = copy.deepcopy(policy_rules)
             policy_rule['hdr_action'] = hdr_action
             if len(policy_action['attrs']) > 3:
-                matches = [policy_action['attrs'][3].replace('\\"',
-                                                            '').replace('"','')]
+                matches = [policy_action['attrs'][3].replace(
+                    '\\"', '').replace('"','')]
                 if matches:
                     value = {'val': matches[0]}
                     policy_rule['hdr_action'][0]['hdr']['value'].update(value)
@@ -927,6 +1026,23 @@ class PolicyConverter(object):
                         policy_name, ns_action_complete_command,
                         STATUS_SUCCESSFUL, policy_rule)
 
+        if policy_action and policy_action['attrs'][1] == 'delete_http_header':
+            hdr_action = [{
+                'action': 'HTTP_REMOVE_HDR',
+                'hdr': {
+                    'name': policy_action['attrs'][2],
+                }
+            }]
+            policy_rule = copy.deepcopy(policy_rules)
+            policy_rule['hdr_action'] = hdr_action
+            LOG.info('Conversion successful: %s %s' % (
+                ns_action_command, policy_name))
+            # Add status successful in CSV/report for policy action
+            ns_util.add_status_row(
+                policy_action['line_no'], ns_action_command,
+                policy_name, ns_action_complete_command,
+                STATUS_SUCCESSFUL, policy_rule)
+
         elif policy_action and policy_action['attrs'][1] == 'replace':
             policy_rule = copy.deepcopy(policy_rules)
             path_matches = \
@@ -1115,6 +1231,16 @@ class PolicyConverter(object):
                 STATUS_SUCCESSFUL, policy_rule)
         elif policy_action and policy_action['attrs'][1] == 'redirect':
             policy_rule = copy.deepcopy(policy_rules)
+            redirect_rule = policy_action['attrs'][2]
+            if '+HTTP.REQ.HOSTNAME+HTTP.REQ.URL.PATH_AND_QUERY' in \
+                redirect_rule:
+                redirect_rule = redirect_rule.replace(
+                    '+HTTP.REQ.HOSTNAME+HTTP.REQ.URL.PATH_AND_QUERY', '')
+                redirect_rule = redirect_rule.replace('"', '')
+                if redirect_rule.endswith('.'):
+                    redirect_rule = redirect_rule[:-1]
+                policy_action['attrs'][2] = redirect_rule
+
             if "\"+" in policy_action['attrs'][2] or "\" +" in \
                     policy_action['attrs'][2]:
                 msg = ("Concatenation in redirect url not supported: %s %s" % (
@@ -1132,8 +1258,12 @@ class PolicyConverter(object):
             redirect_url = ns_util.parse_url(redirect_url)
             protocol = str(redirect_url.scheme).upper()
             protocol = enable_ssl and 'HTTPS' or 'HTTP' if not protocol else \
-                        protocol
-            hostname = str(redirect_url.hostname)
+                        protocol \
+
+            hostname = None
+            if redirect_url.hostname:
+                hostname = str(redirect_url.hostname)
+
             pathstring = str(redirect_url.path)
             querystring = str(redirect_url.query)
             full_path = '%s?%s' % (pathstring, querystring) if pathstring and \
@@ -1177,7 +1307,7 @@ class PolicyConverter(object):
         elif policy_action and policy_action['attrs'][1] == 'respondwith':
             policy_rule = copy.deepcopy(policy_rules)
             attrs = policy_action['attrs'][2].split(' ')
-            if attrs[0].startswith('q{'):
+            if len(attrs) < 2 or attrs[0].startswith('q{'):
                 return
             if attrs[1] == '301':
                 redirect_url = str(attrs[4]).replace('"', '')
